﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;


namespace ConsoleApplication2
{

    public class ORACLE_AlertLogReader: IDisposable 
    {
        private const String CRLF = "\r\n";

        private int LAST_BYTES; 

        public int Last_bytes 
        {
            get
            {
                return this.LAST_BYTES;
            }
            set
            {
                this.LAST_BYTES = value;
            }
        }

        public ORACLE_AlertLogReader()
        {
            this.LAST_BYTES = 0;
        }
        
        private bool Is_header(String line)
        {
            String mask = line.Length >= 4 ? line.Substring(0, 4) : "";
            switch (mask)
            {
                case "Mon ":
                case "Tue ":
                case "Wen ":
                case "Thu ":
                case "Fri ":
                case "Sat ":
                case "Sun ":
                    return true;
                default:
                    return false;
            }
        }

        private bool Is_error(String line)
        {
            return line.IndexOf("error", StringComparison.OrdinalIgnoreCase) > -1;
        }

        public String Read_error_log(String filename)
        {
            String result = null;
            using (StreamReader reader = new StreamReader(filename))
            {
                // read and check last log lines ...
                if (reader.BaseStream.Length > this.LAST_BYTES)
                {
                    reader.BaseStream.Seek(-this.LAST_BYTES, SeekOrigin.End);
                }
                                
                String line = null;
                String block = null;
                while ((line = reader.ReadLine()) != null)
                {
                    if (this.Is_header(line))
                    {   
                        // look up for an error in the block ...
                        if (block.Length > 0)
                        { 
                            if (this.Is_error(block))
                                result += (block + ORACLE_AlertLogReader.CRLF);
                            block = null;
                        }
                    }
                    block += (line + ORACLE_AlertLogReader.CRLF);
                }
                if (block.Length > 0 && this.Is_error(block))
                    result += (block + ORACLE_AlertLogReader.CRLF);
            }

            return result;
        }

        public String Read_error_logs(string[,] LOG_FILES)
        {
            String output = null;
            String dbname = null;
            String logfile = null;
            String dboutput = String.Empty;
            for (int i = 0; i < LOG_FILES.GetLength(0); i++)
            {
                dbname = LOG_FILES[i, 0];
                logfile = LOG_FILES[i, 1];
                dboutput = this.Read_error_log(logfile);
                if (dboutput != null)
                {
                    output += (dbname + ORACLE_AlertLogReader.CRLF + dboutput);
                }
            }
            return output;
        }
    
        public void  Dispose()
        {
 	        //throw new NotImplementedException();
        }
    }

    class Program
    {
        static void Main(string[] args)
        {
            string[,] LOG_FILES = new String[,] {
                                                    {"DB1","C:\\bendak\\applications\\C#_app\\Read_error_log\\alert_db1.log"}, 
                                                    {"DB2","C:\\bendak\\applications\\C#_app\\Read_error_log\\alert_db2.log"}
                                                };
            String output = null;
            using (ORACLE_AlertLogReader alr = new ORACLE_AlertLogReader())
            {
                alr.Last_bytes = 1024 * 2; // read from 2kB from the end of the log file ...
                output = alr.Read_error_logs(LOG_FILES);                
            }

            if (output == null)
                output = "No errors in any DB logs in last 2kB ...";

            Console.WriteLine(output);
        }
    }
}
